@@ -4,6 +4,7 @@ import json
 import logging
 import sys
 import time
+import traceback
 
 import aiohttp
 import requests
@@ -14,6 +15,15 @@ from .cog import Cog
 from .objects import Message, ChatActions, UserProfilePicture, CallbackQuery, Command
 
 
+class CallbackError(Exception):
+    def __init__(self, message=None, *args):
+        if message is not None:
+            self.message = message
+            super().__init__(message, *args)
+        else:
+            super().__init__(*args)
+
+
 class Bot:
 
     def __init__(self, prefixs, user_id, user_hash, **kwargs):
@@ -30,7 +40,7 @@ class Bot:
         self.prefix = prefixs
         self.__cogs = {}
         self.group_photos = []
-        self.actions_from_cog = []
+        self.actions_from_cog = {}
         self.command_roles = {}
         self.ignore_listener_filter = []
         self.message_handlers = {}
@@ -119,7 +129,7 @@ class Bot:
         if not message.get('ok'):
             for _m in self.listeners_handle.get('on_invoke_command_error'):
                 if _m in self.actions_from_cog:
-                    await _m(self, message)
+                    await _m(self.actions_from_cog.get(_m), message)
                 else:
                     await _m(message)
             return None
@@ -164,7 +174,7 @@ class Bot:
     async def dispacth_query(self, query):
         for _m in self.listeners_handle.get('on_callback_query'):
             if _m in self.actions_from_cog:
-                await _m(self, query)
+                await _m(self.actions_from_cog.get(_m), query)
             else:
                 await _m(query)
 
@@ -173,14 +183,18 @@ class Bot:
         return message.date > date
 
     async def handleMessage(self, obj):
-        self.offset = obj.get('update_id') + 1
-        if 'message' in obj:
-            message = Message(self, obj.get('message'))
-            if await self.check_date(message):
-                return await self.dispatch(message)
-        elif 'callback_query' in obj:
-            query = CallbackQuery(self, obj.get('callback_query'))
-            return await self.dispacth_query(query)
+        if len(obj)>=1:
+            self.offset = obj.get('update_id') + 1
+            if 'message' in obj:
+                message = Message(self, obj.get('message'))
+                if await self.check_date(message):
+                    return await self.dispatch(message)
+            elif 'callback_query' in obj:
+                query = CallbackQuery(self, obj.get('callback_query'))
+                try:
+                    return await self.dispacth_query(query)
+                except Exception:
+                    traceback.print_exc()
 
     def has_prefix(self, message):
         if message.text:
@@ -195,13 +209,13 @@ class Bot:
             for _m in self.listeners_handle.get('on_message_new'):
                 if _m in self.ignore_listener_filter or message.chat.id in self.chat_filter:
                     if _m in self.actions_from_cog:
-                        await _m(self, message)
+                        await _m(self.actions_from_cog.get(_m), message)
                     else:
                         await _m(message)
 
     async def dispatch_command(self, message, command):
         if command in self.actions_from_cog:
-            await command.func(self, message)
+            await command.func(self.actions_from_cog.get(command), message)
         else:
             await command.func(message)
 
@@ -210,7 +224,7 @@ class Bot:
             for _m in self.listeners_handle.get('on_unknow_command'):
                 if _m in self.ignore_listener_filter or message.chat.id in self.chat_filter:
                     if _m in self.actions_from_cog:
-                        await _m(self, message)
+                        await _m(self.actions_from_cog.get(_m), message)
                     else:
                         await _m(message)
 
@@ -219,7 +233,7 @@ class Bot:
             for _m in self.listeners_handle.get('on_command_error'):
                 if _m in self.ignore_listener_filter or message.chat.id in self.chat_filter:
                     if _m in self.actions_from_cog:
-                        await _m(self, message, command)
+                        await _m(self.actions_from_cog.get(_m), message, command)
                     else:
                         await _m(message, command)
 
@@ -228,7 +242,7 @@ class Bot:
         if len(self.group_photos) > 1:
             for _m in self.listeners_handle.get('on_group_photo'):
                 if _m in self.actions_from_cog:
-                    await _m(self, message, self.group_photos)
+                    await _m(self.actions_from_cog.get(_m), message, self.group_photos)
                 else:
                     _m(message, self.group_photos)
         else:
@@ -236,7 +250,7 @@ class Bot:
                 for _m in self.listeners_handle.get('on_photo'):
                     if _m in self.ignore_listener_filter or message.chat.id in self.chat_filter:
                         if _m in self.actions_from_cog:
-                            await _m(self, message)
+                            await _m(self.actions_from_cog.get(_m), message)
                         else:
                             await _m(message)
         self.group_photos = []
@@ -249,7 +263,7 @@ class Bot:
             for _m in self.listeners_handle.get('on_sticker_new'):
                 if _m in self.ignore_listener_filter or message.chat.id in self.chat_filter:
                     if _m in self.actions_from_cog:
-                        await _m(self, message)
+                        await _m(self.actions_from_cog.get(_m), message)
                     else:
                         await _m(message)
 
@@ -258,7 +272,7 @@ class Bot:
             for _m in self.listeners_handle.get('on_new_member'):
                 if _m in self.ignore_listener_filter or message.chat.id in self.chat_filter:
                     if _m in self.actions_from_cog:
-                        await _m(self, message)
+                        await _m(self.actions_from_cog.get(_m), message)
                     else:
                         await _m(message)
 
@@ -266,7 +280,7 @@ class Bot:
         if self.listeners_handle.get('on_chat_filter'):
             for _m in self.listeners_handle.get('on_chat_filter'):
                 if _m in self.actions_from_cog:
-                    await _m(self, message)
+                    await _m(self.actions_from_cog.get(_m), message)
                 else:
                     await _m(message)
 
@@ -314,10 +328,10 @@ class Bot:
                 if 'has_arts' in dir(v):
                     command.has_arts = v.has_arts
                 self.add_command(v.__command__, command)
-                self.actions_from_cog.append(command)
+                self.actions_from_cog.update({command:cls})
             elif '__listener__' in dir(v):
                 self.add_listener(v.__listener__, v)
-                self.actions_from_cog.append(v)
+                self.actions_from_cog.update({v:cls})
                 if v.__ignore_filter__:
                     self.ignore_listener_filter.append(v)
 
@@ -336,15 +350,19 @@ class Bot:
                     if 'on_pre_command' in self.listeners_handle:
                         for _m in self.listeners_handle.get('on_pre_command'):
                             if _m in self.actions_from_cog:
-                                await _m(self, rs, message)
+                                await _m(self.actions_from_cog.get(_m), rs, message)
                             else:
                                 await _m(rs, message)
                     if len(self.chat_filter) > 0:
                         if message.chat.id not in self.chat_filter and rs not in self.ignore_filter:
                             return await self.dispatch_chat_filter_error(message)
                     await self.dispatch_command(message, rs)
-                except Exception:
-                    await self.dispatch_error_command_invoke(message, rs)
+                except CallbackError as e:
+                    await message.reply(e.message)
+                    return  await self.dispatch_error_command_invoke(message, rs)
+                except Exception as t:
+                    traceback.print_exc()
+                    return await self.dispatch_error_command_invoke(message, rs)
             else:
                 await self.dispatch_uknow_command(message)
         await self.dispatch_message(message)
@@ -374,7 +392,7 @@ class Bot:
         if 'on_start' in self.listeners_handle:
             for _m in self.listeners_handle.get('on_start'):
                 if _m in self.actions_from_cog:
-                    await _m(self)
+                    await _m(self.actions_from_cog.get(_m))
                 else:
                     await _m()
         while True:
