@@ -17,7 +17,7 @@ class Command:
 
 class CallbackQuery:
     def __init__(self, bot, payload):
-        print(payload)
+
         self.id = payload.get('id')
         self.user = User(payload.get('from'))
         self.message = Message(bot, payload.get('message'))
@@ -82,6 +82,7 @@ class Message:
         self.edit_date = datetime.datetime.fromtimestamp(payload.get('edit_date')) if 'edit_date' in payload else None
         self.new_chat_member = User(payload.get('new_chat_member')) if 'new_chat_member' in payload else None
         self.media_group_id = payload.get('media_group_id') if 'media_group_id' in payload else -1
+        self.entities = [Entity(p) for p in payload.get('entities')] or []
 
 
     async def delete_message(self):
@@ -117,13 +118,13 @@ class Message:
             rs = await self.bot.tg_request('editMessageText', True, **data)
             return Message(self.bot, rs.get('result'))
 
-    async def reply(self, text, reply_markup=None, photo=None, **kwargs):
+    async def reply(self, text, reply_markup=None, photo=None,parse_mode=None, **kwargs):
         data = {
             'chat_id': self.chat.id or kwargs['chat_id'],
             'reply_to_message_id': self.message_id
         }
-        if kwargs['parse_mode']:
-            data['parse_mode'] = kwargs['parse_mode']
+        if parse_mode:
+            data['parse_mode'] = parse_mode
         if reply_markup:
             data['reply_markup'] = json.dumps(reply_markup.to_dict())
         if photo:
@@ -132,11 +133,15 @@ class Message:
             return await self.send_photo(photo,**data)
 
         data['text'] = text
-        print(data)
+        if self.chat.id != self.message_id:
+            del data["reply_to_message_id"]
         rs = await self.bot.tg_request('sendMessage', True, **data)
         return rs.get('ok')
 
 
+
+
+
 class Sticker:
     def __init__(self, payload):
         self.width = payload.get('width')
@@ -194,6 +199,14 @@ class User:
     def get_full_name(self):
         return self.first_name + ' ' + self.last_name
 
+class Entity(User):
+    def __init__(self, payload):
+        super().__init__(payload.get('user'))
+        self.offset = payload.get('offset')
+        self.length = payload.get('length')
+        self.type = payload.get('type')
+
+
 
 class UserChat:
     def __init__(self, payload):
